<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;

class LockScreenMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure(\Illuminate\Http\Request): (\Illuminate\Http\Response|\Illuminate\Http\RedirectResponse)  $next
     * @return \Illuminate\Http\Response|\Illuminate\Http\RedirectResponse
     */
    public function handle($request, Closure $next)
    {
        $response = $next($request);

        // Check if user is authenticated
        if (Auth::check()) {
            $lastActivityTime = session('lastActivityTime');

            // Check if user has been inactive for more than 10 minutes
            if ($lastActivityTime && now()->diffInMinutes($lastActivityTime) >= env('SESSION_LIFETIME')) {
                Auth::logout();
                $request->session()->invalidate();

                // Redirect to the login page
                return Redirect::route('login')->with('message', 'You have been logged out due to inactivity.');
            }

            // Store last activity time in session
            $request->session()->put('lastActivityTime', now());
        }

        return $response;
    }
}
